import Image, ImageDraw
import urllib2
import sys

from api.navigation import Route, Point

KEY = 'AIzaSyCIIdJRlM8Iu_ldTW4j9HdqRsSUhL0H5GQ'
HELP_TEXT = """Usage: python dominant_color.py <filename> <num_of_colors>
<maximum_distance_between_points> 

filename -- PNG file to save the graphic.

num_of_colors -- Number of dominant colors to process.

maximum_distance_between_points -- Maximum distance between route points in
meters.

camera_heading -- Heading of camera relative to vehicle.

"""
GRAPHIC_BLOCK_SIZE = 40

def save_graphic(images, filename, num_of_colors):
    width = num_of_colors * GRAPHIC_BLOCK_SIZE
    height = len(images) * GRAPHIC_BLOCK_SIZE
    img = Image.new("RGB", (width, height))
    draw = ImageDraw.Draw(img)
    for i, image in enumerate(images):
        colors = image.dominant_colors(KEY, num_of_colors, y_offset=200)
        for j, color in enumerate(colors):
            draw.rectangle(
                (j * GRAPHIC_BLOCK_SIZE,
                 i * GRAPHIC_BLOCK_SIZE,
                 j * GRAPHIC_BLOCK_SIZE + GRAPHIC_BLOCK_SIZE,
                 i * GRAPHIC_BLOCK_SIZE + GRAPHIC_BLOCK_SIZE),
                fill=color.hex_code()
            )
        print 'Frame %s: %s' % \
                (i + 1, [' '.join([color.hex_code() for color in colors])])
    img.save(filename, 'PNG')

def main():
    try:
        filename = sys.argv[1]
        num_of_colors = int(sys.argv[2])
        maximum_distance_between_points = int(sys.argv[3])
        camera_heading = int(sys.argv[4])
    except:
        print HELP_TEXT
        return -1
    origin = raw_input('Origin address: ')
    origin_point = Point.for_address(origin)
    destination = raw_input('Destination address: ')
    destination_point = Point.for_address(destination)
    route = Route.between(origin_point, destination_point,
                          maximum_distance_between_points)
    save_graphic(route.google_street_view_images(heading=camera_heading),
                 filename, num_of_colors)

if __name__ == '__main__':
    main()
